"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const ts = require("typescript/lib/tsserverlibrary");
const config_1 = require("./config");
const analysis_1 = require("./analysis");
const utils_1 = require("./utils");
const getTemplateExpressions = (node) => ts.isTemplateExpression(node)
    ? node.templateSpans.map(span => span.expression)
    : [];
const stringifyParameter = (parameter) => [
    parameter.schema,
    parameter.table,
    parameter.column,
    parameter.jsonPath && parameter.jsonPath.path
]
    .filter(x => x)
    .join(".");
const getParameterType = (parameter, schemaJson, defaultSchemaName) => {
    const schema = parameter.schema || defaultSchemaName;
    const dbSchema = schemaJson[schema];
    const dbTable = dbSchema && dbSchema[parameter.table];
    const dbColumn = dbTable && dbTable[parameter.column];
    if (dbColumn) {
        const type = parameter.jsonPath && parameter.jsonPath.isText
            ? "string | null"
            : dbColumn;
        return parameter.isArray ? `Array<${type}>` : type;
    }
};
const getDiagnosticFactory = (context) => {
    const file = context.node.getSourceFile();
    const source = config_1.pluginName;
    const error = ({ code, messageText, start, length }) => ({
        code,
        messageText,
        category: context.typescript.DiagnosticCategory.Error,
        file,
        start,
        length,
        source
    });
    const errorAtExpression = (expression, { code, messageText }) => error({
        code,
        messageText,
        start: expression.getStart(file) - context.node.getStart(file) - 1,
        length: expression.getWidth(file)
    });
    return {
        error,
        errorAtExpression
    };
};
class SqlTemplateLanguageService {
    constructor(logger, config, typeChecker) {
        this.logger = logger;
        this.config = config;
        this.typeChecker = typeChecker;
    }
    getSemanticDiagnostics(context) {
        const factory = getDiagnosticFactory(context);
        let analysis;
        try {
            analysis = analysis_1.analyze(context.text);
        }
        catch (e) {
            if (e instanceof analysis_1.ParseError) {
                return [
                    factory.error({
                        code: 1001,
                        messageText: `Failed to parse: ${e.message}`,
                        start: e.cursorPosition - 1,
                        length: 1
                    })
                ];
            }
            else {
                return [
                    factory.error({
                        code: 1002,
                        messageText: `Failed to parse: ${e.message}`,
                        start: 0,
                        length: context.text.length
                    })
                ];
            }
        }
        for (const warning of analysis.warnings) {
            this.logger.log(`Warning analyzing template in file ${context.fileName}: ${warning}`);
        }
        const schema = this.config.schema;
        if (!schema) {
            return [];
        }
        const expressions = getTemplateExpressions(context.node);
        const diagnostics = Array.from(analysis.parameters.entries())
            .map(([index, parameter]) => ({
            expression: expressions[index - 1],
            parameter
        }))
            .map(({ expression, parameter }) => {
            const parameterType = getParameterType(parameter, schema, this.config.defaultSchemaName);
            if (!parameterType) {
                return [
                    factory.errorAtExpression(expression, {
                        code: 1003,
                        messageText: `Cannot find type for parameter ${stringifyParameter(parameter)} in schema`
                    })
                ];
            }
            const expressionType = this.typeChecker.getType(expression);
            const content = `{ let expr: ${expressionType}; let param: ${parameterType} = expr; }`;
            return this.typeChecker.check(content).map(diagnostic => factory.errorAtExpression(expression, {
                code: 1004,
                messageText: diagnostic.messageText
            }));
        });
        return utils_1.flatten(diagnostics);
    }
}
exports.default = SqlTemplateLanguageService;
//# sourceMappingURL=language-service.js.map